let socket = io()

//////////////////// Set-up the Visual ////////////////////

//10-bit value that can come in from the phototransistor
const max_value = 1023

//Create SVG
const width = 500
const height = 850

const svg = d3.select("#chart").append("svg")
    .attr("width", width)
    .attr("height", height)
    .attr("class", "svg-chart")

//Create scales
const scale_color_wb = d3.scaleLinear()
    .domain([500, 950])
    .range(["white", "#0d0f1d"])
    .clamp(true)

const scale_color_bw = d3.scaleLinear()
    .domain([500, 950])
    .range(["#0d0f1d", "white"])
    .clamp(true)

const colors = ["#66489f", "#ff33a5", "#efb605"].reverse()
const scale_color_bars = d3.scaleLinear()
    .domain(d3.range(colors.length).map(d => { return d/(colors.length-1) * max_value}))
    .range(colors)
    .clamp(true)

///////////////////// Background color ////////////////////

const rect_background = svg.append("rect")
    .attr("class", "background")
    .attr("x", 0)
    .attr("y", 0)
    .attr("width", width)
    .attr("height", height)
    .style("opacity", 0.7)

function updateBackground(data) {
    rect_background.style("fill", scale_color_bars(data))
}//function updateBackground

////////////////// Text on the background /////////////////

const text_value = svg.append("text")
    .attr("class", "background-text")
    .attr("transform", `translate(${width/2}, ${height/2})`)
    .style("text-anchor", "middle")
    .style("font-size", "170px")

function updateText(data) {
    text_value
        .style("fill", "white") //scale_color_wb(data))
        .text(data)
}//function updateText

//////////////////////// Bar chart ////////////////////////
const bar_group = svg.append("g")
    .attr("class", "bar-group")
    .attr("transform", `translate(${width/2}, ${0})`)

const bar_width = 25
const bar_height = 6
const bar_corner = bar_height/2

function updateBars(data) {
    let num_bars = Math.round(data/10)

    //Append new data to the group
    let bars = bar_group.selectAll(".bar")
        .data(d3.range(num_bars), d => d)

    //EXIT
    bars.exit().remove() 

    //ENTER - Group
    let enter_bars = bars.enter()
        .append("rect")
        .attr("class", "bar")
        .attr("id", d => `bar-${d}`)
        .attr("x", -bar_width/2)
        .attr("width", bar_width)
        .attr("height", bar_height)
        .attr("rx", bar_corner)
        .style("fill", d => scale_color_bars(d * 10))

    //MERGE
    bars = enter_bars.merge(bars)

    /////////////////////// ENTER + UPDATE //////////////////////
    bars
        .attr("y", d => height - d * (bar_height + 2))
}//function updateBars


///////////////////////// Get Data ////////////////////////
//Get the data from the serial port and update the visual
socket.on('connect', function () {
    socket.on('parsed-data', function (data) {
        //console.log(data)

        //Update the visual
        updateBackground(data)
        updateText(data)
        updateBars(data)
    })//socket.on
})//socket.on
